<?php

namespace Bitrix\KMsending\bxMessageServices;

use Bitrix\KMsending\Config\Service;
use Bitrix\Main\Error;
use Bitrix\MessageService\Sender\Base;
use Bitrix\MessageService\Sender\Result\SendMessage;
use Bitrix\KMsending\Sendings\Sender;
use Bitrix\KMsending\Debug;
use Bitrix\KMsending\Exceptions\KMsendingException;
use \Bitrix\Main\Config\Option;
use \Bitrix\Main\Localization\Loc;


/**
 * Class KMsending - Отправка сообщений через службу сообщений Битрикса
 *
 * @package KMsending\bxMessageServices
 */
class kmsending extends Base
{

    private $KMsending;

    /**
     * KMsending constructor.
     */
    public function __construct()
    {
        $this->KMsending = new \CKMsending();
    }

    /**
     * @return string
     */
    public function getShortName()
    {
        return 'KMsending';
    }

    /**
     * @return string
     */
    public function getId()
    {
        return 'KMsending';
    }

    /**
     * @return mixed
     * @throws \KMsending\Exceptions\KMsendingException
     */
    public function getName()
    {
        return Loc::getMessage('KMSENDING_TMS_CONTENT');
    }

    /**
     * @param array $messageFields
     *
     * @return \Bitrix\MessageService\Sender\Result\SendMessage
     * @throws \KMsending\Exceptions\KMsendingException
     * @throws \KMsending\Sendings\Exceptions\KMsendingValidationError
     */
    public function sendMessage(array $messageFields)
    {

        $sender = new Sender();
        $result = new SendMessage();
        $debug = new Debug\Service();
        try {
            $sender->bxMessageServicesSendSms([$messageFields['MESSAGE_TO'] => $messageFields['MESSAGE_BODY']], $messageFields['MESSAGE_FROM']??'');
        } catch (KMsendingValidationError $e) {
            $result->addErrors($e->getMessage());
            throw new KMsendingException(Loc::getMessage('KMSENDING_VALIDATION_ERROR'), $e->getCode(), $e);
        }

        return $result;
    }

    /**
     * @return bool
     */
    public function canUse()
    {
        return true;
    }

    /**
     * @return array
     * @throws \KMsending\Exceptions\KMsendingException
     */
    public function getFromList()
    {
        try {
            if ($this->isReadyToWorkModuleKMsending()) {
                $addresses = $this->KMsending->getAllSenders();
                $arAddr = [];
                foreach ($addresses as $addr) {
                    $arAddr[] = [
                        'id'   => $addr,
                        'name' => $addr
                    ];
                }

                return $arAddr;
            }
        } catch (KMsendingException $e) {
            $debug = new Debug\Service();
            $debug->writeToLogFile($e->getMessage() . ' ' . $e->getFile() . ' ' . $e->getLine() . PHP_EOL);
        }
    }

    /**
     * @return bool
     * @throws \KMsending\Exceptions\KMsendingException
     */
    public function isReadyToWorkModuleKMsending(): bool
    {
        $config = new Service();
        $addresses = $this->KMsending->getAllSenders();

        if (\CModule::IncludeModule('kmsending')
            && Option::get("kmsending", "module_enabled") === 'Y'
            && !empty($config->getToken())
            && !empty($addresses[0])
        ) {
            return true;
        }
        return false;
    }
}
