<?php
declare(strict_types=1);

namespace Bitrix\KMsending\Incoming;

use Bitrix\Main\Localization\Loc;
use Bitrix\Main\UI\PageNavigation;
use Bitrix\KMsending\RemoteService;
use Bitrix\Main\EventManager;
use Bitrix\KMsending\Exceptions\KMsendingException;
use Bitrix\KMsending\Config;
use Bitrix\KMsending\Address;
use \KMsending\Exceptions\KMsendingRepositoryException;

class Service
{
    /**
     * @var RemoteService\Service
     */
    private $remoteService;
    /**
     * @var Repository
     */
    private $repository;
    /**
     * @var Config\Service
     */
    private $config;
    /**
     * @var Address\Service
     */
    private $addressService;

    public function __construct()
    {
        $this->remoteService = new RemoteService\Service();
        $this->repository = new Repository();
        $this->config = new Config\Service();
        $this->addressService = new Address\Service();
    }

    /**
     * @throws KMsendingException
     */
    public function processLoadIncoming(): void
    {
        try {
            if ($this->config->getModuleEnableSetting() === 'Y' && $this->addressService->hasAccountDigitNumber()) {
                $lastIncomingLoad = new \DateTime($this->config->getLoadIncomingFrom());
                $response = $this->remoteService->LoadSMS($lastIncomingLoad, 1);

                if ($response->isSuccess()) {
                    foreach ($response->getLoadSmsCollection() as $loadSms) {
                        $incoming = Incoming::createInstanceFromDto($loadSms);
                        $this->repository->save($incoming);

                        $lastIncomingLoad = $incoming->getMoment();
                        $lastIncomingLoad->modify('+1 second');

                        $incomingForEvent = clone $incoming;

                        $arEvents = EventManager::getInstance()->findEventHandlers('kmsending', 'LoadIncoming');
                        foreach ($arEvents as $arEvent) {
                            ExecuteModuleEventEx($arEvent, [$incomingForEvent]);
                        }
                    }
                    $this->config->setLoadIncomingFrom($lastIncomingLoad);
                } else {
                    throw new KMsendingException(Loc::getMessage('KMSENDING_REMOTE_SERVICE_ERROR',
                        ['#CODE#' => $response->getState()->getCode()]));
                }
            }
        } catch (\Exception $e) {
            throw new KMsendingException($e->getMessage(), $e->getCode(), $e);
        }
    }

    /**
     * @param array               $filter
     * @param PageNavigation|null $nav
     * @param array               $order
     *
     * @return Collection
     * @throws KMsendingException
     */
    public function getIncomingByFilter(array $filter, ?PageNavigation $nav = null, array $order = []): Collection
    {
        try {
            return $this->repository->getByFilter($filter, $nav, $order);
        } catch (KMsendingRepositoryException $e) {
            throw new KMsendingException($e->getMessage(), $e->getCode(), $e);
        }
    }
}