<?php
declare(strict_types=1);

namespace Bitrix\KMsending\RemoteService\MethodExecutor;

use Bitrix\Main\Localization\Loc;
use Bitrix\KMsending\Exceptions\KMsendingException;

abstract class MethodExecutor
{
    private $api_url = 'https://1-pulse.kupon.mobi/api/v1/bulk';
    protected $client;

    /**
     * @throws KMsendingException
     */
    public function __construct()
    {
        try {
            $this->client = curl_init();
            curl_setopt( $this->client, CURLOPT_AUTOREFERER, TRUE);
            curl_setopt( $this->client, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt( $this->client, CURLOPT_FOLLOWLOCATION, TRUE);
            curl_setopt( $this->client, CURLOPT_HEADER, false);
            curl_setopt( $this->client, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt( $this->client, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt( $this->client, CURLOPT_POST, 1);

        } catch (\Exception $e) {
            curl_close($this->client);
            throw new KMsendingException(Loc::getMessage('KMSENDING_MAIN_ERROR_CONNECT'));
        }
    }

    /**
     * @return string
     */
    abstract protected function getMethodName(): string;


    /**
     * @param array $requestArguments
     *
     * @return array
     */
    protected function prepareRequestArguments(array $requestArguments): array
    {
        return $requestArguments;
    }

    /**
     * @param string $methodName
     * @param array  $requestArguments
     *
     * @return mixed
     * @throws KMsendingException
     */
    final protected function performRequest(string $methodName, array $requestArguments)
    {
        try{
            curl_setopt( $this->client, CURLOPT_URL, $this->api_url.'/'.$methodName);
            curl_setopt( $this->client, CURLOPT_POSTFIELDS, $requestArguments);


            $response = curl_exec($this->client);

            return $response;
        } catch (\Exception $e) {
            curl_close($this->client);
            throw new KMsendingException(Loc::getMessage('KMSENDING_MAIN_CALL_METHOD_ERROR',
                ['#METHOD#' => $methodName]), $e->getCode(), $e->getPrevious());
        }
    }

    /**
     *
     * @param $rawResponse
     *
     */
    abstract protected function prepareResponse($rawResponse);

    /**
     * @param array $requestArguments
     *
     * @throws KMsendingException
     */

    /**
     * @param array $requestArguments
     *
     * @return mixed
     * @throws \KMsending\Exceptions\KMsendingException
     */
    final public function execute(array $requestArguments=[])
    {
        $requestArguments = $this->prepareRequestArguments($requestArguments);
        $rawResponse = $this->performRequest($this->getMethodName(), $requestArguments);

        if (curl_getinfo($this->client, CURLINFO_HTTP_CODE) == 200) {
            curl_close($this->client);
            return $this->prepareResponse($rawResponse);
        } else {
            throw new KMsendingException(Loc::getMessage('KMSENDING_MAIN_CALL_METHOD_ERROR',
                ['#METHOD#' => $this->getMethodName()]), curl_errno($this->client));
            curl_close($this->client);
        }
    }

}