<?php
declare(strict_types=1);

namespace Bitrix\KMsending\RemoteService\Methods\SendListMessages;

use Bitrix\Main\Localization\Loc;
use Bitrix\KMsending\Config\Service;
use Bitrix\KMsending\Exceptions\KMsendingException;
use Bitrix\KMsending\RemoteService;
use Bitrix\KMsending\Sendings;

class MethodExecutor extends RemoteService\MethodExecutor\MethodExecutor
{
    private const PHONE_PATTERN = "/^[+]?[78]?[(]?([93]{1}\d{2})[)]?(\d{7})$/";

    /**
     * array keys
     */
    const MESSAGES = 'messages';
    const SENDER = 'sender';
    const OFF = 'Off';
    const START = 'Start';
    const GROUP = 'Group';
    const ENCODING = 'Encoding';
    const BODY = 'Body';

    /**
     * @var string
     */
    private $dateFormat = 'Y-m-d H:i:s';
    private $list;

    /**
     * {@inheritdoc}
     */
    protected function getMethodName(): string
    {
        return 'send-list-messages';
    }

    /**
     * {@inheritdoc}
     */
    protected function prepareResponse($rawResponse)
    {
        $response = json_decode($rawResponse, true);
        $group = time();

        $smsResultCollection = new RemoteService\Dto\GroupSms\Collection();
        /**
         * @var Sendings\Messages\Message $message
         */
        $group_result=1;
        foreach ($this->list as $key=>$message) {
                $smsResult = new RemoteService\Dto\GroupSms\SmsResult(
                    $message->getGuid(),
                    ($response[$key]['type']!='success'?$response[$key]['type'].': '.$response[$key]['msg']:'1')
                );
            $smsResultCollection->attach($smsResult);

        }
        if(!is_array($response) || $response['type']=='error') $group_result=-30;
           return new Response($group_result, $group, $smsResultCollection);

    }

    /**
     * @inheritDoc
     */
    protected function prepareRequestArguments(array $requestArguments): array
    {
        $requestArguments[self::MESSAGES] = $this->prepareList($requestArguments[self::MESSAGES],$requestArguments[self::SENDER] );
        $requestArguments[self::OFF] = $this->prepareDateTime($requestArguments[self::OFF]);
        $requestArguments[self::START] = $this->prepareDateTime($requestArguments[self::START]);
        unset($requestArguments[self::SENDER]);
        return $requestArguments;
    }

    /**
     * @param Sendings\SendingInQueue\Message\Collection $list
     *
     * @return array
     */
    private function prepareList(Sendings\SendingInQueue\Message\Collection $list, $sender): string
    {
        $sender=explode(':', $sender);
        if(count($sender)!=2) {
            throw new KMsendingException(Loc::getMessage('KMSENDING_GROUP_SMS_RESPONSE_30'), -30);
        }
        $type=trim($sender[0]);
        $sender=trim($sender[1]);
        $this->list=$list;
        $messageParameters = [];
        $config = new Service();
        /**
         * @var Sendings\SendingInQueue\Message\Collection $message
         */
        foreach ($list as $message) {
            $messageParameters[] = [
                'sender' => $sender,
                'type' => $type,
                'sent_to' => $message->getDestination(),
                'sent_text' => $message->getText(),
                'callback'=>$config->getCallBackHost().'/personal/kmsending/notification.php?guid='. $message->getGuid() .'&status={status}',
                'find_token'=> ((preg_match(self::PHONE_PATTERN, $message->getDestination()) || filter_var( $message->getDestination(), FILTER_VALIDATE_EMAIL)) && in_array($type, ['bot', 'push']))?'1':'0'
            ];
        }

        return json_encode($messageParameters);
    }

    /**
     * @param \DateTime|null $dateTime
     *
     * @return string
     */
    private function prepareDateTime(?\DateTime $dateTime): string
    {
        if (!$dateTime instanceof \DateTime) {
            return '';
        }

        return $dateTime->format($this->dateFormat);
    }

}