<?php
declare(strict_types=1);

namespace Bitrix\KMsending\RemoteService;

use Bitrix\Main\Localization\Loc;
use CEvent;
use CModule;
use COption;
use Bitrix\Main\Mail;
use CSite;
//use Bitrix\KMsending\RemoteService\Methods\StartSession;
//use Bitrix\KMsending\RemoteService\Methods\CloseSession;
//use Bitrix\KMsending\RemoteService\Methods\GroupSMS;
//use Bitrix\KMsending\RemoteService\Methods\CheckSMS;
//use Bitrix\KMsending\RemoteService\Methods\ParamSMS;
use Bitrix\KMsending\RemoteService\Methods\LoadSMS;
use Bitrix\KMsending\RemoteService\Methods\Balance;
use Bitrix\KMsending\RemoteService\Methods\Senders;
use Bitrix\KMsending\RemoteService\Methods\SendListMessages;
use Bitrix\KMsending\Sendings;
use Bitrix\KMsending\Config;
use Bitrix\KMsending\Exceptions\KMsendingException;

class Service
{
//    private const BASE_CONNECTION = 'session_id_base';
//    private const CONNECTION_FOR_SEND = 'session_id_send_sms';
    /**
     * @var string
     */
    private $token;
    /**
     * @var int
     */
    private $gmt;
    /**
     * @var int|null
     */
//    private $sessionId;
    /**
     * @var string|null
     */
    public $lastError;
    /**
     * @var Config\Service
     */
    private $config;

    /**
     * @throws KMsendingException
     */
    public function __construct()
    {
        $this->config = new Config\Service();
        $this->token=$this->config->getToken();
//        $this->sessionId = $this->config->getSessionId(self::BASE_CONNECTION);
    }

//    /**
//     * @param string $login
//     * @param string $password
//     * @param int    $gmt
//     *
//     * @return StartSession\Response
//     * @throws KMsendingException
//     */
//    private function StartSession(string $login, string $password, int $gmt): StartSession\Response
//    {
//        $instance = new StartSession\MethodExecutor();
//        $response = $instance->execute([
//            'Login'    => $login,
//            'Password' => $password,
//            'Gmt'      => $gmt
//        ]);
//
//        return $response;
//    }

//    /**
//     * @param int $sessionId
//     *
//     * @return CloseSession\Response
//     * @throws KMsendingException
//     */
//    private function CloseSession(int $sessionId): CloseSession\Response
//    {
//        $instance = new CloseSession\MethodExecutor();
//        $response = $instance->execute([
//            'SessionID' => $sessionId,
//        ]);
//        return $response;
//    }

//    /**
//     * @param int|null                                   $group
//     * @param string                                     $sender
//     * @param int|null                                   $encoding
//     * @param string|null                                $body
//     * @param \DateTime|null                             $actualDate
//     * @param \DateTime|null                             $startDate
//     * @param string                                     $period
//     * @param Sendings\SendingInQueue\Message\Collection $list
//     *
//     * @return GroupSMS\Response
//     * @throws KMsendingException
//     */
//    public function GroupSMS(
//        ?int $group,
//        string $sender,
//        ?int $encoding,
//        ?string $body,
//        ?\DateTime $actualDate,
//        ?\DateTime $startDate,
//        string $period,
//        Sendings\SendingInQueue\Message\Collection $list
//    ): GroupSMS\Response {
//        $instance = new GroupSMS\MethodExecutor();
//        $response = $instance->execute([
//            'SessionId' => $this->sessionId,
//            'Group'     => $group,
//            'Source'    => $sender,
//            'Encoding'  => $encoding,
//            'Body'      => $body,
//            'Off'       => $actualDate,
//            'Start'     => $startDate,
//            'Period'    => $period,
//            'List'      => $list,
//        ]);
//        return $response;
//    }
//
//    /**
//     * @param array $guids
//     *
//     * @return CheckSMS\Response
//     * @throws KMsendingException
//     */
//    public function CheckSMS(
//        array $guids
//    ): CheckSMS\Response {
//        $instance = new CheckSMS\MethodExecutor();
//        $response = $instance->execute([
//            'SessionId' => $this->sessionId,
//            'Guids'     => $guids
//        ]);
//        return $response;
//    }

//    /**
//     * @return ParamSMS\Response
//     * @throws KMsendingException
//     */
//    public function ParamSMS(): ParamSMS\Response
//    {
//        $instance = new ParamSMS\MethodExecutor();
//        $response = $instance->execute([
//            'SessionId' => $this->sessionId
//        ]);
//        return $response;
//    }

    /**
     * @return Balance\Response
     * @throws KMsendingException
     */
    public function Balnace(): Balance\Response
    {
        $instance = new Balance\MethodExecutor();
        $response = $instance->execute([
            'token'=> $this->token
        ]);
        return $response;
    }

    /**
     * @return Balance\Response
     * @throws KMsendingException
     */
    public function Senders(): Senders\Response
    {
        $instance = new Senders\MethodExecutor();
        $response = $instance->execute([
            'token'=> $this->token
        ]);
        return $response;
    }

    public function SendListMessages(
        string $sender,
        ?int $encoding,
        ?\DateTime $actualDate,
        ?\DateTime $startDate,
        Sendings\SendingInQueue\Message\Collection $list
    ): SendListMessages\Response {
        $instance = new SendListMessages\MethodExecutor();
        $response = $instance->execute([
            'token'=> $this->token,
            'sender'    => $sender,
            'Encoding'  => $encoding,
            'Off'       => $actualDate,
            'Start'     => $startDate,
            'messages'      => $list,
        ]);
        return $response;
    }
    /**
     * @param \DateTime $changesFrom
     * @param int       $flags
     *
     * @return LoadSMS\Response
     * @throws KMsendingException
     */
    public function LoadSMS(
        \DateTime $changesFrom,
        int $flags
    ): LoadSms\Response {
        $instance = new LoadSMS\MethodExecutor();
        $response = $instance->execute([
            'SessionId'   => $this->sessionId,
            'ChangesFrom' => $changesFrom,
            'Flags'       => $flags
        ]);
        return $response;
    }

    /**
     * @param int|null    $gmt
     *
     * @throws KMsendingException
     */
    public function openConnectToService(
        ?int $gmt = null
    ): void {
        $this->gmt = $gmt ?? (int)$this->config->getGmt();
        if (empty($this->token)) {
            throw new KMsendingException(Loc::getMessage('KMSENDING_NO_TOKEN'));
        }
    }


//    /**
//     * @param string $variableName
//     *
//     * @throws KMsendingException
//     */
//    private function createSessionId(string $variableName): void
//    {
//        $response = $this->StartSession($this->login, $this->password, $this->gmt);
//
//        if ($response->isSuccess()) {
//            $this->config->setSessionId($variableName, $response->getState()->getCode());
//            $this->sessionId = $response->getState()->getCode();
//            $this->config->setSentErrorEmailSetting('N');
//            $this->config->clearDateForSendEmail();
//        } else {
//            $this->sendEmailIsNeeded();
//            $this->lastError = $response->getState()->getDescription();
//        }
//    }

//    /**
//     * @param string $variableName
//     *
//     * @throws KMsendingException
//     */
//    private function updateSessionId(string $variableName): void
//    {
//        if (!empty($this->password) && !empty($this->login)) {
//            if ($this->sessionId === null || $this->sessionId === 0) {
//                $this->createSessionId($variableName);
//            } else {
//                if (!$this->ParamSMS()->isSuccess()) {
//                    $this->createSessionId($variableName);
//                }
//            }
//        } else {
//            throw new KMsendingException(Loc::getMessage('KMSENDING_NO_TOKEN'));
//        }
//    }

    /**
     * @return string|null
     */
    public function getLastError(): ?string
    {
        return $this->lastError;
    }

    /**
     * @param string $lastError
     */
    public function setLastError(string $lastError): void
    {
        $this->lastError = $lastError;
    }

    /**
     * @return void
     * @throws KMsendingException
     */
    private function sendEmailIsNeeded(): void
    {
        if ($this->config->getSendEmailToAdminSetting() === 'Y' &&
            $this->config->getSentErrorEmailSetting() !== 'Y'
        ) {
            $now = new \DateTime('now');
            $lastCheckDate = $this->config->getDateForSendEmail();
            if (empty($lastCheckDate)) {
                $this->config->setDateForSendEmail($now->add(new \DateInterval('PT5M')));
            } else {
                try {
                    $lastCheckDate = new \DateTime($lastCheckDate);
                } catch (\Exception $e) {
                    return;
                }
                if ($now > $lastCheckDate) {
                    if (CEvent::SendImmediate('KMSENDING_ADMIN_SEND', CSite::GetDefSite(),
                            ['DEFAULT_EMAIL_FROM' => COption::GetOptionString('main', 'email_from')])
                        === Mail\Event::SEND_RESULT_SUCCESS
                    ) {
                        $this->config->setSentErrorEmailSetting('Y');
                    }
                }

            }

        }
    }

}